"use strict";
/**
 *  Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of
 *  https://github.com/graphql/codemirror-graphql repository. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var CodeMirror = require("codemirror/addon/runmode/runmode.node");
var graphql_language_service_parser_1 = require("graphql-language-service-parser");
/**
 * The GraphQL mode is defined as a tokenizer along with a list of rules, each
 * of which is either a function or an array.
 *
 *   * Function: Provided a token and the stream, returns an expected next step.
 *   * Array: A list of steps to take in order.
 *
 * A step is either another rule, or a terminal description of a token. If it
 * is a rule, that rule is pushed onto the stack and the parsing continues from
 * that point.
 *
 * If it is a terminal description, the token is checked against it using a
 * `match` function. If the match is successful, the token is colored and the
 * rule is stepped forward. If the match is unsuccessful, the remainder of the
 * rule is skipped and the previous rule is advanced.
 *
 * This parsing algorithm allows for incremental online parsing within various
 * levels of the syntax tree and results in a structured `state` linked-list
 * which contains the relevant information to produce valuable typeaheads.
 */
CodeMirror.defineMode("graphql", function (config) {
    var parser = graphql_language_service_parser_1.onlineParser({
        eatWhitespace: function (stream) { return stream.eatWhile(graphql_language_service_parser_1.isIgnored); },
        lexRules: graphql_language_service_parser_1.LexRules,
        parseRules: graphql_language_service_parser_1.ParseRules,
        editorConfig: { tabSize: config.tabSize },
    });
    return {
        config: config,
        startState: parser.startState,
        token: parser.token,
        indent: function (state, textAfter) {
            var levels = state.levels;
            // If there is no stack of levels, use the current level.
            // Otherwise, use the top level, pre-emptively dedenting for close braces.
            var level = !levels || levels.length === 0
                ? state.indentLevel
                : levels[levels.length - 1] -
                    (this.electricInput.test(textAfter) ? 1 : 0);
            return level * this.config.indentUnit;
        },
        electricInput: /^\s*[})\]]/,
        fold: "brace",
        lineComment: "#",
        closeBrackets: {
            pairs: '()[]{}""',
            explode: "()[]{}",
        },
    };
});
